package prupp;

// $Id: Config.java,v 1.3 1998/12/19 13:46:09 d94-rol Exp $
// $Author: d94-rol $
// $Date: 1998/12/19 13:46:09 $

import java.io.*;
import java.util.*;

/** a class to handle configs for different things.
 *  reads file on the format <pre>
 *  [sectionName1]
 *  key=value
 *  key2=value2
 *  [sectionName2]
 *  key=value
 *  key2=value2 </pre>
 *  everything after the first '#' is considered a comment.
 *  blank lines are ignored.
 */
public class Config {
    private Hashtable configs;

    /** create an empty Config (has only section "" with no data in it)
     */
    public Config () {
	configs = new Hashtable ();
	Properties current = new Properties ();   // the main thing.    
	configs.put ("", current); 
    }

    /** create a Config for the specified file
     * @param filename the File we read the config from
     */
    public Config (String filename) throws IOException {
	BufferedReader br = new BufferedReader (new FileReader (filename));

	String line;
	configs = new Hashtable ();
	Properties current = new Properties ();   // the main thing.    
	configs.put ("", current); 
	while ((line = br.readLine ()) != null) {
	    int index = line.indexOf ('#');
	    if (index >= 0) 
		line = line.substring (0,index);
	    if (line.equals (""))
		continue;
	    if (line.startsWith ("[")) {
		int endindex = line.indexOf (']');
		if (endindex >= 0) {
		    String newSection = line.substring (1,endindex);
		    // collapse configs named equal.
		    if (configs.get (newSection) != null)
			current = (Properties)configs.get (newSection);
		    else 
			current = new Properties ();
		    configs.put (newSection, current);	  
		}
	    }
	    else {
		int eqindex = line.indexOf ('=');
		String key = "" , value = "";
		if (eqindex >= 0) {
		    key = line.substring (0,eqindex);
		    value = line.substring (eqindex+1);
		} else 
		    key = line;
		current.put (key, value);
	    }
	}
	br.close ();
    }

    /** get the available sections
     * @return an Enumeration of the available sections (including the empty section).
     */
    public Enumeration getSections () {
	return configs.keys ();
    }

    /** get the properties for a given section
     * @param sectionName the section we want properties for.
     * @return a Properties if section exist or null.
     */
    public Properties getProperties (String sectionName) {
	return (Properties)configs.get (sectionName);
    }
    
    /** set the properties for a given section
     * @param sectionName the section we want to set the properties for.
     * @param prop the Properties for the sections
     */
    public void setProperties (String sectionName, Properties prop) {
	configs.put (sectionName, prop);
    }

    /** get a property for given key in specified section
     * @param section the section we should look in.
     * @param key the key we want a value for.
     * @return a string if section + key is set, null otherwise
     */
    public String getProperty (String section, String key) {
	Properties p = getProperties (section);
	if (p != null) 
	    return p.getProperty (key);
	return null;
    }

    /** set a property for given section.
     * @param section the section we should look in.
     * @param key the key.
     * @param value the value.
     */
    public void setProperty (String section, String key, String value) {
	Properties p = getProperties (section);
	if (p == null) {
	    p = new Properties ();
	    configs.put (section, p);
	}
	p.put (key, value);
    }
    
    /** save the config to a OutputStream 
     */
    public void save (OutputStream os) {
	save (os, null);
    }
    /** save the config to a OutputStream 
     */
    public void save (OutputStream os, String comment) {
	PrintWriter dos = new PrintWriter (os);
	if (comment != null) {
	    StringTokenizer st = new StringTokenizer (comment, "\n");
	    while (st.hasMoreElements ()) {
		String str = (String)st.nextElement ();
		dos.println ("#" + str);
	    }
	}
	Enumeration e = configs.keys ();
	while (e.hasMoreElements ()) {
	    String section = (String)e.nextElement ();
	    dos.println ("[" + section + "]");
	    Properties pr = (Properties)configs.get (section);
	    Enumeration ek = pr.keys ();
	    while (ek.hasMoreElements ()) {
		String key = (String)ek.nextElement ();
		String value = pr.getProperty (key, "");
		dos.println (key + "=" + value);
	    }
	    dos.println ("");
	}
	dos.flush ();
    }
}
