package prupp;

// $Id: Logger.java,v 1.1.1.1 1998/08/10 21:13:36 d94-rol Exp $

import java.util.*;

/** this class handles logging of events.
 *  Events are classified by there string-represantion.
 */
public class Logger 
{
    private Hashtable counters;
    private Vector viewers;

    /** the interface to implement when you want to snoop on the log */
    public interface LogViewer
    {
	/** Invoked when the log has changed
	 */
	public void setChanged ();
    }

    class LogEntry 
    {
	String entryname;
	int counter;
	LogEntry (String name)
	    {
		entryname = name;
		counter = 0;
	    }
	void inc () 
	    {
		counter++;
	    }
	public int count () 
	    {
		return counter;
	    }	
    }
    
    /** default constructor 
     */
    Logger () 
	{
	    counters = new Hashtable ();
	    viewers = new Vector ();
	}

    /** Add a viewer to this logger
     * @param lv the LogViewer to add 
     */
    public void addViewer (LogViewer lv)
	{
	    viewers.addElement (lv);
	}

  
    /** increase a logentry.
     * @param log the event to increase 
     */
    public void inc (String log) 
	{
	    LogEntry l = (LogEntry)(counters.get(log));
	    if (l == null) {
		l = new LogEntry (log);
		counters.put (log,l);
	    }
	    l.inc ();

	    for (int i = 0; i < viewers.size (); i++)
		((LogViewer)viewers.elementAt (i)).setChanged ();
	}

    /** get all events
     * @return an Enumeration of all events
     */
    public Enumeration keys () 
	{
	    return counters.keys ();
	}
  
    /** get all values (unsorted)
     * @return an Enumeration of all values
     */
    public Enumeration values ()
	{
	    return counters.elements ();
	}
    
    /** get the current count for an event.
     * @param key the event were intrested in
     * @return the current count of event.
     */
    public int get (String key) 
	{
	    LogEntry l = (LogEntry)counters.get (key);
	    if (l == null)
		return 0;
	    else 
		return l.count ();
	}
}


