package prupp;

// $Id$

import java.io.*;
import java.net.*;
import java.util.*;
import java.lang.reflect.*;

/** A MetaPage to get a value in the configurations.
 *  This page handles four very distinct sections:
 *  proxy, access, filter and users.
 */
public class OneGetter implements MetaHandler
{
    /** default constructor 
     */
    public OneGetter () {
    }
    
    /** Get one value from the settings and return it to the client.
     * @param input the InputStream from the client.
     * @param output the OutputStream to the client.
     * @param url the requested url.
     * @param htab the supplied argument to the page (CGI-parameters).
     * @param uap the username and paswword as by Basic-authentication
     */
    public void handle (DataInputStream input, DataOutputStream output, URL url, Properties htab, String uap) {

	HTTPHeader header = Proxy.getHeader ();
	header.setHeader ("Content-type", "text/plain");
	StringBuffer res = new StringBuffer (header.toString ());
	String section = htab.getProperty ("section");
	String filter = htab.getProperty ("filter");
	String setting = htab.getProperty ("setting");
	String error = null;

	if (section == null) {
	    res.append ("Specify section");
	}
	else if (section.equals ("proxy")) {
	    if (setting != null) {
		String value = Proxy.getProperties ().getProperty (setting);
		if (value == null)
		    error = "Setting does not exist";
		else 
		    res.append (value);
	    }
	    else 
		error = "No setting specified";
	}
	else if (section.equals ("filter")) {
	    if (setting != null && filter != null) {
		if (filter.indexOf (".") < 0)
		    filter = "prupp." + filter;
		try {
		    // is this insecure? you can now configure any class....
		    // this is kinda bad or is it good?
		    Class cls = Class.forName (filter);  // throws exception unless found.
		    Class params[] = new Class[0];
		    Method getprops = cls.getDeclaredMethod ("getProperties", params);
		    Object arguments[] = new Object [0];
		    Properties prop = (Properties)getprops.invoke (cls, arguments);
		    String value =  prop.getProperty (setting);
		    if (value == null)
			error = "Setting does not exist";
		    else 
			res.append (value);
		} catch (ClassNotFoundException e) {
		    error = "Couldnt find " + filter;
		} catch (NoSuchMethodException e) {
		    error = "couldnt find method getProperties for: " + filter + " (this is probably nothing to worry about)";
		} catch (SecurityException e) {
		    error = "Seems reflection aint allowed.";
		} catch (InvocationTargetException e) {
		    error = "couldnt invoke getProperties for " + filter + 
			", nested exception is:" + e.getTargetException ();
		} catch (IllegalAccessException e) {
		    error = e.toString ();
		}
		if (error != null) {
		    error = "Setup failed: " + error;
		    Proxy.logError (error);
		}
	    } else 
		error = "both filter and section needs to be set.";
	}
	else if (section.equals ("access")) {
	    Enumeration e = Proxy.getAccesses ().elements ();
	    while (e.hasMoreElements ()) {
		Access acc = (Access) e.nextElement ();
		res.append (acc.toString () + "\n");
	    }
	}
	else if (section.equals ("users")) {
	    Hashtable users = Proxy.getUsers ();
	    Enumeration e = users.keys ();
	    while (e.hasMoreElements ()) {
		String uname = (String)e.nextElement ();
		String pass = (String)users.get (uname);
		res.append (uname + ":" + pass + "\n");
	    }	    
	}
	
	res.append ((error != null ? "ERROR: " + error : ""));
	try {
	    output.write (res.toString ().getBytes ());
	} catch (IOException e) {
	    Proxy.logError ("Couldnt write status");
	}
    }
}
