package prupp.html;

import java.util.*;

/** A class to handle a block of text and split it 
 *  into a text/tags - vector.
 */
public class HTMLBlock 
{
    private Vector tags = new Vector ();
    private String pagepart = null;
    private String rest = null;

    /** create a new HTMLBlock with the given page.
     * @param pagepart the block of data to handle.
     */
    public HTMLBlock (String pagepart) {
	this.pagepart = pagepart;
	parse ();
    }
    
    /** build the internal representation
     */
    protected void parse () {
	int i = 0;
	int j = 0;
	int tagstart = -1;
	int tagend = -1;
	int lasttagend = 0;
	HTMLTag html;

	while (i < pagepart.length () && i >= 0) {
	    i = pagepart.indexOf ('<', i);
	    if (i < 0) {
		tags.addElement (pagepart.substring (lasttagend));
		break;
	    }
	    if (lasttagend != i) {
		tags.addElement (pagepart.substring (lasttagend,i));
	    }
	    tagstart = i++;
	    j = findwhite (i);
	    if (j < 1) {
		rest = pagepart.substring (tagstart);
		break;
	    }
	    
	    /** ignore thoose bad html that put code in non-escaped form. */
	    if (j == i) {
		i++;
		continue;
	    }

	    /** ARGH!!!! bad html suck. anyway try to get nice tokens code is ignored. */
	    char c = pagepart.charAt (i);
	    if ((c!='!'&&c!='/')&&(!((c>='a'&&c<='z')||(c>='A'&&c<='Z')))) {
		i++;
		continue;
	    } 

	    String tag = pagepart.substring (i,j);

	    if (tag.startsWith ("!--")) {
		int endcomment =  getCommentTag (i); 
		if (endcomment < 0) {
		    rest = pagepart.substring (tagstart);
		    break;
		}
		html = new HTMLTag (pagepart.substring (i+3,endcomment),true); // the two '-' but not the '>'
		tagend = endcomment + 2;
	    }
	    else {
		tag = tag.toUpperCase ();
		html = new HTMLTag (tag);
		tagend = setAttributes (html, j);
		if (tagend < 0) {
		    rest = pagepart.substring (tagstart);
		    break;
		}
	    }
	    tags.addElement (html);
	    i = lasttagend = tagend+1;
	}
    }

    /** find the index of a "white" that is something that ends a token
     * @param i the index to search from.
     * @return the index to the white-char.
     */
    protected int findwhite (int i) {
	while (i < pagepart.length ()) {
	    switch (pagepart.charAt (i)) {
	    case ' ':
	    case '\t':
	    case '\r':
	    case '\n':
	    case '>':
		return i;
	    default:
		i++;
	    }
	}
	return -1;
    }

    /** Get the index of the next nonwhite token.
     * @param i the index to search from
     * @return the index of the next non-white token
     */
    protected int nextNonWhite (int i) {
	while (i < pagepart.length ()) {
	    switch (pagepart.charAt (i)) {
	    case ' ':
	    case '\t':
	    case '\r':
	    case '\n':
		i++;
		break;
	    default:
		return i;
	    }
	}
	return -1;
    }

    /** Get the index of the next separator character.
     * @param i the index to search from
     * @return the index of the next separator character.
     */
    protected int findSeparator (int i) {
	while (i < pagepart.length ()) {
	    switch (pagepart.charAt (i)) {
	    case ' ':
	    case '\t':
	    case '\r':
	    case '\n':
	    case '>':
	    case '=':
		return i;
	    default:
		i++;
	    }
	}
	return -1;	
    }

    /** Sets all attributes for a htmltag.
     * @param start the index to start getting tokens from.
     * @return the end index of the tag.
     */
    protected int setAttributes (HTMLTag html, int start) {
	String currentToken = null;
	String nextToken = "";
	int i = start,j;

	do {
	    j = nextNonWhite (i);
	    if (j < 0)
		return j;
	    /*System.out.println ("found: " + pagepart.charAt(j) + pagepart.charAt(j+1) + " " +
	      "currentToken: " + currentToken + " nextToken: " + nextToken);*/
	    switch (pagepart.charAt (j)) {
	    case '>':
		nextToken = ">";
		i = j;
		break;
	    case '"':
	    case '\'':
		int ec = pagepart.indexOf (pagepart.charAt (j),j+1);
		if (ec < 0)
		    return ec;
		if (currentToken != null)
		    html.setAttribute (currentToken);
		nextToken = currentToken = pagepart.substring (j,ec+1);
		i = ec+1;
		break;
	    case '=':
		int startval = nextNonWhite (j+1);
		if (startval < 0)
		    return startval;
		switch (pagepart.charAt (startval)) {
		case '>':
		    html.setAttribute (currentToken, "");
		    i = startval+1;
		    nextToken = ">";
		    break;
		case '"':
		case '\'':
		    int ev = pagepart.indexOf (pagepart.charAt(startval), startval+1);
		    if (ev < 0)
			return ev;
		    nextToken = pagepart.substring (startval+1,ev);
		    html.setAttribute (currentToken, nextToken);
		    i = ev+1;
		    break;
		default:
		    int ea = findwhite (startval);
		    if (ea < 0)
			return ea;
		    nextToken = pagepart.substring (startval,ea);
		    html.setAttribute (currentToken, nextToken);
		    i = ea;
		}
		currentToken = null;
		break;
	    default:
		int ea = findSeparator (j);
		if (ea < 0)
		    return ea;
		nextToken = pagepart.substring (j,ea);
		if (currentToken != null)
		    html.setAttribute (currentToken);
		currentToken = nextToken;
		i = ea;
	    }
	} while (!nextToken.equals (">"));
	if (currentToken != null)
	    html.setAttribute (currentToken);
	return i;
    }    

    /** Get the index of where the comment ends. 
     * @param i the index to start searching from.
     * @return the index of "-->".
     */
    protected int getCommentTag (int i) {
	return pagepart.indexOf ("-->", i);
    }

    /** Get the string representation of this object.
     * @return a string describing this object.
     */
    public String toString () {
	StringBuffer sb = new StringBuffer ();
	
	for (int i = 0; i < tags.size (); i++){
	    String c = tags.elementAt (i).toString ();
	    sb.append (c);
	}
	
	return sb.toString ();
    }

    /** Get the Vector of elements (the strings and tags).
     * @return a Vector of String and HTMLTag objects.
     */
    public Vector getElements () {
	return tags;
    }

    /** Get the rest of the block. That is the part which couldnt be parsed (non-ended tags).
     * @return a string with the part that couldnt be parsed.
     */
    public String getRest () {
	return rest;
    }

    /** Simple testprogram for this class.
     */
    public static void main (String args[]) {
	String test = "You can read about <a HrEF=\"comments.htm\">what others have said about our site</A>. Find\nout how to <A HREF=\"webmaster.htm\">add a link</A> to our site if you have your own homepage. \nYou can also find what has been <A HREF=\"../new.htm\">added recently</A>. If you can't decide\nwhat to read <A HREF=\"/scripts/randdoc.plx\">choose one at random</A>.\n\n<P>\n\nIf you have any suggestions\n";

	String test2 = "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\"><html>\n<head>\n<!--title>Menu</title>\n</head-->\n\n<body Background=\"pics/bg1.jpg\" bgcolor=\"black\" text=\"#FFe0c0\" \nlink=\"#ffff00\" vlink=\"#00ff00\" alink=\"ffffff\">\n\n<table border=0 height=\"100%\">\n<!--\nThe header, same for all pages!\n-->\n<DIV CLASS=\"top\">\n\n\n<apa adf";

	HTMLBlock hb = new HTMLBlock (test2);
	System.out.println ("hb is: \n" + hb);
	System.out.println ("rest is: " + hb.rest + "\n");
	System.out.println ("test is: \n" + test2);
    }
}

